{% extends 'uniauth/base-site.html' %}

<!-- Page-specific headers -->
{% block head %}

<title>Uniauth | Reset</title>

<style>
</style>

{% endblock %}

<!-- Page Content -->
{% block content %}

<div id="password-reset-form-spacer" class="spacer"></div>

<div id="password-reset-form-wrapper" class="uniauth-wrapper wrapper wrapper-content">
    <div class="row">
        <div class="col-lg-12">
            <div class="ibox float-e-margins shadow-lg">
                <div class="ibox-content">

                    <div class="row">
                        <div class="col-lg-12">
                            <span id="password-reset-title" class="simple-title centered-title">Reset Your Password</span>
                        </div>
                    </div>

                    <div class="row">
                        <div class="col-lg-12">
                            <span id="password-reset-email-label" class="simple-followup">Enter your email address, and we'll send you a link to reset your password.</span>
                            <form method="POST" class="post-form">
                                {% csrf_token %}

                                {% if form.non_field_errors %}
                                <div class="row">
                                    <div class="col-lg-12">
                                        <div class="form-group has-error">
                                            <span id="error-text" class="form-error help-block">
                                                {{ form.non_field_errors.as_text|slice:"2:" }}
                                            </span>
                                        </div>
                                    </div>
                                </div>
                                {% endif %}

                                <div class="row">
                                    <div class="col-lg-12">
                                        <input id="id_email" name="email" type="email" placeholder="Email Address" class="form-control input-block" value="{{ form.username.value|default_if_none:"" }}" autocapitalize="off" autocorrect="off" autofocus required />
                                    </div>
                                </div>

                                <button id="submit-btn" type="submit" class="save btn btn-action">Send reset email</button>
                            </form>
                        </div>
                    </div>

                </div>
            </div>
        </div>
    </div>
</div>

{% endblock %}

<!-- Page specific JS -->
{% block script %}
{% endblock %}
